@@ -3,7 +3,7 @@
 readone <- function(ln, iwd, nper, conv)
 {
     ln <- gsub("D", "E", ln)
-    inds <- seq(0, by = iwd, length = nper + 1)
+    inds <- seq(0, by = iwd, length.out = nper + 1)
     (conv)(substring(ln, 1 + inds[-length(inds)], inds[-1]))
 }
 
@@ -36,9 +36,9 @@ readHB <- function(file)
         on.exit(close(file))
     }
     hdr <- readLines(file, 4, ok = FALSE)
-    Title <- sub('[[:space:]]+$', '', substr(hdr[1], 1, 72))
-    Key <- sub('[[:space:]]+$', '', substr(hdr[1], 73, 80))
-    totln <- as.integer(substr(hdr[2], 1, 14))
+    ## Title <- sub('[[:space:]]+$', '', substr(hdr[1], 1, 72))
+    ## Key   <- sub('[[:space:]]+$', '', substr(hdr[1], 73, 80))
+    ## totln <- as.integer(substr(hdr[2], 1, 14))
     ptrln <- as.integer(substr(hdr[2], 15, 28))
     indln <- as.integer(substr(hdr[2], 29, 42))
     valln <- as.integer(substr(hdr[2], 43, 56))
@@ -54,14 +54,12 @@ readHB <- function(file)
     nr <- as.integer(substr(hdr[3], 15, 28))
     nc <- as.integer(substr(hdr[3], 29, 42))
     nz <- as.integer(substr(hdr[3], 43, 56))
-    nel <- as.integer(substr(hdr[3], 57, 70))
+    ## nel <- as.integer(substr(hdr[3], 57, 70))
     ptrfmt <- toupper(sub('[[:space:]]+$', '', substr(hdr[4], 1, 16)))
     indfmt <- toupper(sub('[[:space:]]+$', '', substr(hdr[4], 17, 32)))
     valfmt <- toupper(sub('[[:space:]]+$', '', substr(hdr[4], 33, 52)))
-    rhsfmt <- toupper(sub('[[:space:]]+$', '', substr(hdr[4], 53, 72)))
-    if (!is.na(rhsln) && rhsln > 0) {
-        h5 <- readLines(file, 1, ok = FALSE)
-    }
+    ## rhsfmt <- toupper(sub('[[:space:]]+$', '', substr(hdr[4], 53, 72)))
+    if (!is.na(rhsln) && rhsln > 0) readLines(file, 1, ok = FALSE) # h5
     ptr <- readmany(file, ptrln, nc + 1, ptrfmt, as.integer)
     ind <- readmany(file, indln, nz, indfmt, as.integer)
     vals <- readmany(file, valln, nz, valfmt, as.numeric)
@@ -87,7 +85,7 @@ readMM <- function(file)
     scan1 <- function(what, ...)
 	scan(file, nmax = 1, what = what, quiet = TRUE, ...)
 
-    if ((hdr <- scan1(character())) != "%%MatrixMarket")
+    if (scan1(character()) != "%%MatrixMarket")# hdr
 	stop("file is not a MatrixMarket file")
     if (!(typ <- tolower(scan1(character()))) %in% "matrix")
 	stop(gettextf("type '%s' not recognized", typ), domain = NA)
@@ -104,10 +102,15 @@ readMM <- function(file)
     nc <- scan1(integer())
     nz <- scan1(integer())
     checkIJ <- function(els) {
-	if(els$i < 1 || els$i > nr)
-	    stop("readMM(): row	 values 'i' are not in 1:nr", call.=FALSE)
-	if(els$j < 1 || els$j > nc)
-	    stop("readMM(): column values 'j' are not in 1:nc", call.=FALSE)
+        if((nz. <- length(els$i)) < nz)
+            warning(gettextf("readMM(): expected %d entries but found only %d",
+                             nz, nz.), call. = FALSE, domain = NA)
+        if(any(is.na(els$i) | els$i < 1L | els$i > nr))
+            stop(gettextf("readMM(): row indices 'i' are not in 1:nrow[=%d]",
+                          nr), call. = FALSE, domain = NA)
+        if(any(is.na(els$j) | els$j < 1L | els$j > nc))
+            stop(gettextf("readMM(): column indices 'j' are not in 1:ncol[=%d]",
+                          nc), call. = FALSE, domain = NA)
     }
     if (repr == "coordinate") {
 	switch(elt,
