@@ -2,8 +2,7 @@
 #define MATRIX_SSC_H
 
 #include "Mutils.h"
-#include "Csparse.h"
-/* -> diag_tC() */
+#include "Csparse.h" /* -> diag_tC() */
 #include "chm_common.h"
 
 SEXP R_chkName_Cholesky(SEXP nm, SEXP perm, SEXP LDL, SEXP super);
@@ -14,6 +13,10 @@ SEXP dsCMatrix_LDL_D(SEXP Ap, SEXP permP, SEXP resultKind);
 SEXP dsCMatrix_chol(SEXP x, SEXP pivot);
 SEXP dsCMatrix_Csparse_solve(SEXP a, SEXP b, SEXP LDL);
 SEXP dsCMatrix_matrix_solve (SEXP a, SEXP b, SEXP LDL);
+
+/* MJ: no longer used ... prefer R_sparse_as_general(), CRsparse_as_Tsparse() */
+#if 0
 SEXP dsCMatrix_to_dgTMatrix(SEXP x);
+#endif /* MJ */
 
 #endif
